@@ -1,10 +1,12 @@
 #include "app.h"
 
-#include <QFileOpenEvent>
-#include <QTextStream>
-#include <QFileInfo>
 #include <QDir>
+#include <QFileInfo>
+#include <QFileOpenEvent>
+#include <QSettings>
+#include <QTextStream>
 #include "mainwindow.h"
+#include "arguments.h"
 
 namespace NeovimQt {
 
@@ -86,11 +88,16 @@ bool App::event(QEvent *event)
 
 void App::showUi(NeovimConnector *c, const QCommandLineParser& parser)
 {
-	auto opts = ShellOptions();
+	ShellOptions opts{ GetShellOptionsFromQSettings() };
+
 	if (parser.isSet("no-ext-tabline")) {
 		opts.enable_ext_tabline = false;
 	}
 
+	if (parser.isSet("no-ext-popupmenu")) {
+		opts.enable_ext_popupmenu = false;
+	}
+
 #ifdef NEOVIMQT_GUI_WIDGET
 	NeovimQt::Shell *win = new NeovimQt::Shell(c);
 	win->show();
@@ -107,6 +114,19 @@ void App::showUi(NeovimConnector *c, const QCommandLineParser& parser)
 	QObject::connect(instance(), SIGNAL(openFilesTriggered(const QList<QUrl>)),
 		win->shell(), SLOT(openFiles(const QList<QUrl>)));
 
+	// Window geometry should be restored only when the user does not specify
+	// one of the following command line arguments. Argument "maximized" can
+	// be safely ignored, as the loaded geometry only takes effect after the
+	// user un-maximizes the window; this behavior is desirable. Function
+	// `isSet` will issue qWarning() messages if the argument doesn't exist.
+	// Do not call isSet(...) for arguments which may not exist.
+	if (!parser.isSet("fullscreen") &&
+		(!hasGeometryArg() || !parser.isSet("geometry")) &&
+		(!hasQWindowGeometryArg() || !parser.isSet("qwindowgeometry")))
+	{
+		win->restoreWindowGeometry();
+	}
+
 	if (parser.isSet("fullscreen")) {
 		win->delayedShow(NeovimQt::MainWindow::DelayedShow::FullScreen);
 	} else if (parser.isSet("maximized")) {
@@ -131,10 +151,21 @@ void App::processCliOptions(QCommandLineParser &parser, const QStringList& argum
 	parser.addOption(QCommandLineOption("timeout",
 				QCoreApplication::translate("main", "Error if nvim does not responde after count milliseconds"),
 				QCoreApplication::translate("main", "ms"),
-				"10000"));
-	parser.addOption(QCommandLineOption("geometry",
-				QCoreApplication::translate("main", "Initial window geometry"),
-				QCoreApplication::translate("main", "geometry")));
+				"20000"));
+
+	// Some platforms use --qwindowgeometry, while other platforms use the --geometry.
+	// Make the correct help message is displayed.
+	if (hasGeometryArg()) {
+		parser.addOption(QCommandLineOption("geometry",
+			QCoreApplication::translate("main", "Set initial window geometry"),
+			QCoreApplication::translate("main", "width>x<height")));
+	}
+	if (hasQWindowGeometryArg()) {
+		parser.addOption(QCommandLineOption("qwindowgeometry",
+			QCoreApplication::translate("main", "Set initial window geometry"),
+			QCoreApplication::translate("main", "width>x<height")));
+	}
+
 	parser.addOption(QCommandLineOption("stylesheet",
 				QCoreApplication::translate("main", "Apply qss stylesheet from file"),
 				QCoreApplication::translate("main", "stylesheet")));
@@ -142,6 +173,8 @@ void App::processCliOptions(QCommandLineParser &parser, const QStringList& argum
 				QCoreApplication::translate("main", "Maximize the window on startup")));
 	parser.addOption(QCommandLineOption("no-ext-tabline",
 				QCoreApplication::translate("main", "Disable the external GUI tabline")));
+	parser.addOption(QCommandLineOption("no-ext-popupmenu",
+				QCoreApplication::translate("main", "Disable the external GUI popup menu")));
 	parser.addOption(QCommandLineOption("fullscreen",
 				QCoreApplication::translate("main", "Open the window in fullscreen on startup")));
 	parser.addOption(QCommandLineOption("embed",
@@ -247,5 +280,31 @@ NeovimConnector* App::createConnector(const QCommandLineParser& parser)
 	}
 }
 
+/*static*/ ShellOptions App::GetShellOptionsFromQSettings()
+{
+	ShellOptions opts{};
+	QSettings settings("nvim-qt", "nvim-qt");
 
-} // Namespace
+	QVariant ext_linegrid{ settings.value("ext_linegrid", opts.enable_ext_linegrid) };
+	QVariant ext_popupmenu{ settings.value("ext_popupmenu", opts.enable_ext_popupmenu) };
+	QVariant ext_tabline{ settings.value("ext_tabline", opts.enable_ext_popupmenu) };
+
+	if (ext_linegrid.canConvert<bool>())
+	{
+		opts.enable_ext_linegrid = ext_linegrid.toBool();
+	}
+
+	if (ext_popupmenu.canConvert<bool>())
+	{
+		opts.enable_ext_popupmenu = ext_popupmenu.toBool();
+	}
+
+	if (ext_tabline.canConvert<bool>())
+	{
+		opts.enable_ext_tabline= ext_tabline.toBool();
+	}
+
+	return opts;
+}
+
+} // namespace NeovimQt
