@@ -1,8 +1,9 @@
 #include "mainwindow.h"
 
 #include <QCloseEvent>
-#include <QToolBar>
 #include <QLayout>
+#include <QSettings>
+#include <QToolBar>
 
 namespace NeovimQt {
 
@@ -49,10 +50,32 @@ void MainWindow::init(NeovimConnector *c)
 	m_tabline_bar->addWidget(m_tabline);
 	m_tabline_bar->setVisible(m_shell_options.enable_ext_tabline);
 
+	// Context menu and actions for right-click
+	m_contextMenu = new QMenu();
+	m_actCut = new QAction(QIcon::fromTheme("edit-cut"), QString("Cut"), nullptr /*parent*/);
+	m_actCopy = new QAction(QIcon::fromTheme("edit-copy"), QString("Copy"), nullptr /*parent*/);
+	m_actPaste = new QAction(QIcon::fromTheme("edit-paste"), QString("Paste"), nullptr /*parent*/);
+	m_actSelectAll = new QAction(QIcon::fromTheme("edit-select-all"), QString("Select All"),
+		nullptr /*parent*/);
+	m_contextMenu->addAction(m_actCut);
+	m_contextMenu->addAction(m_actCopy);
+	m_contextMenu->addAction(m_actPaste);
+	m_contextMenu->addSeparator();
+	m_contextMenu->addAction(m_actSelectAll);
+
 	m_nvim = c;
+
+	m_tree = new TreeView(c);
 	m_shell = new Shell(c, m_shell_options);
-	m_stack.insertWidget(1, m_shell);
+
+	m_window = new QSplitter();
+	m_window->addWidget(m_tree);
+	m_tree->hide();
+	m_window->addWidget(m_shell);
+
+	m_stack.insertWidget(1, m_window);
 	m_stack.setCurrentIndex(1);
+
 	connect(m_shell, SIGNAL(neovimAttached(bool)),
 			this, SLOT(neovimAttachmentChanged(bool)));
 	connect(m_shell, SIGNAL(neovimTitleChanged(const QString &)),
@@ -79,6 +102,16 @@ void MainWindow::init(NeovimConnector *c)
 			this, &MainWindow::neovimTablineUpdate);
 	connect(m_shell, &Shell::neovimShowtablineSet,
 			this, &MainWindow::neovimShowtablineSet);
+	connect(m_shell, &Shell::neovimShowContextMenu,
+			this, &MainWindow::neovimShowContextMenu);
+	connect(m_actCut, &QAction::triggered,
+			this, &MainWindow::neovimSendCut);
+	connect(m_actCopy, &QAction::triggered,
+			this, &MainWindow::neovimSendCopy);
+	connect(m_actPaste, &QAction::triggered,
+			this, &MainWindow::neovimSendPaste);
+	connect(m_actSelectAll, &QAction::triggered,
+			this, &MainWindow::neovimSendSelectAll);
 	m_shell->setFocus(Qt::OtherFocusReason);
 
 	if (m_nvim->errorCause()) {
@@ -143,7 +176,19 @@ void MainWindow::neovimWidgetResized()
 	// widget size - this avoids situations when neovim wants a size that
 	// exceeds the available widget size i.e. the GUI tells neovim its
 	// size, not the other way around.
-	m_shell->resizeNeovim(m_shell->size());
+	if (isMaximized() || isFullScreen()) {
+		QSize size = geometry().size();
+        if (m_tabline_bar->isVisible()) {
+            size.setHeight(size.height() - m_tabline_bar->geometry().size().height());
+        }
+		if (m_tree->isVisible()) {
+			size.scale(size.width() - m_tree->geometry().size().width(),
+				size.height(), Qt::IgnoreAspectRatio);
+		}
+		m_shell->resizeNeovim(size);
+	} else {
+		m_shell->resizeNeovim(m_shell->size());
+	}
 }
 
 void MainWindow::neovimMaximized(bool set)
@@ -187,6 +232,12 @@ void MainWindow::reconnectNeovim()
 
 void MainWindow::closeEvent(QCloseEvent *ev)
 {
+	// Do not save window geometry in '--fullscreen' mode. If saved, all
+	// subsequent Neovim-Qt sessions would default to fullscreen mode.
+	if (!isFullScreen()) {
+		saveWindowGeometry();
+	}
+
 	if (m_neovim_requested_close) {
 		// If this was requested by nvim, shutdown
 		QWidget::closeEvent(ev);
@@ -242,8 +293,13 @@ void MainWindow::showIfDelayed()
 void MainWindow::neovimAttachmentChanged(bool attached)
 {
 	emit neovimAttached(attached);
-	if (isWindow() && m_shell != NULL) {
-		m_shell->updateGuiWindowState(windowState());
+	if (attached) {
+		if (isWindow() && m_shell != NULL) {
+			m_shell->updateGuiWindowState(windowState());
+		}
+	} else {
+		m_tabline->deleteLater();
+		m_tabline_bar->deleteLater();
 	}
 }
 
@@ -292,6 +348,7 @@ void MainWindow::neovimTablineUpdate(int64_t curtab, QList<Tab> tabs)
 			m_tabline->setTabText(index, text);
 		}
 
+		m_tabline->setTabToolTip(index, text);
 		m_tabline->setTabData(index, QVariant::fromValue(tabs[index].tab));
 
 		if (curtab == tabs[index].tab) {
@@ -311,6 +368,31 @@ void MainWindow::neovimTablineUpdate(int64_t curtab, QList<Tab> tabs)
 	Q_ASSERT(tabs.size() == m_tabline->count());
 }
 
+void MainWindow::neovimShowContextMenu()
+{
+	m_contextMenu->popup(QCursor::pos());
+}
+
+void MainWindow::neovimSendCut()
+{
+	m_nvim->api0()->vim_command_output(R"(normal! "+x)");
+}
+
+void MainWindow::neovimSendCopy()
+{
+	m_nvim->api0()->vim_command(R"(normal! "+y)");
+}
+
+void MainWindow::neovimSendPaste()
+{
+	m_nvim->api0()->vim_command(R"(normal! "+gP)");
+}
+
+void MainWindow::neovimSendSelectAll()
+{
+	m_nvim->api0()->vim_command("normal! ggVG");
+}
+
 void MainWindow::changeTab(int index)
 {
 	if (!m_shell_options.enable_ext_tabline) {
@@ -324,5 +406,19 @@ void MainWindow::changeTab(int index)
 	int64_t tab = m_tabline->tabData(index).toInt();
 	m_nvim->api2()->nvim_set_current_tabpage(tab);
 }
-} // Namespace
 
+void MainWindow::saveWindowGeometry()
+{
+	QSettings settings{ "nvim-qt", "window-geometry" };
+	settings.setValue("window_geometry", saveGeometry());
+	settings.setValue("window_state", saveState());
+}
+
+void MainWindow::restoreWindowGeometry()
+{
+	QSettings settings{ "nvim-qt", "window-geometry" };
+	restoreGeometry(settings.value("window_geometry").toByteArray());
+	restoreState(settings.value("window_state").toByteArray());
+}
+
+}  // namespace NeovimQt
