@@ -104,13 +104,13 @@ bool Shell::setGuiFont(const QString& fdesc, bool force)
 		return false;
 	}
 
-	int pointSize = font().pointSize();
+	qreal pointSize = font().pointSizeF();
 	int weight = -1;
 	bool italic = false;
 	foreach(QString attr, attrs) {
 		if (attr.size() >= 2 && attr[0] == 'h') {
 			bool ok = false;
-			int height = attr.mid(1).toInt(&ok);
+			qreal height = attr.mid(1).toFloat(&ok);
 			if (!ok) {
 				m_nvim->api0()->vim_report_error("Invalid font height");
 				return false;
@@ -149,6 +149,9 @@ void Shell::setAttached(bool attached)
 		if (isWindow()) {
 			updateGuiWindowState(windowState());
 		}
+
+		updateClientInfo();
+
 		auto req_shim = m_nvim->api0()->vim_command("runtime plugin/nvim_gui_shim.vim");
 		connect(req_shim, &MsgpackRequest::error, this, &Shell::handleShimError);
 		auto gviminit = qgetenv("GVIMINIT");
@@ -159,23 +162,12 @@ void Shell::setAttached(bool attached)
 			m_nvim->api0()->vim_command(gviminit);
 		}
 
-		updateClientInfo();
-
 		// Noevim was not able to open urls till now. Check if we have any to open.
 		if(!m_deferredOpen.isEmpty()){
 			openFiles(m_deferredOpen);
 			m_deferredOpen.clear();    //Neovim may change state. Clear to prevent reopening.
 		}
 
-		// Show v:errmsg if available
-		auto api1 = m_nvim->api1();
-		auto req = api1->nvim_get_vvar("errmsg");
-		connect(req, &MsgpackRequest::finished, [api1](quint32 m, quint64 f, const QVariant& r) {
-			auto err = r.toString();
-			if (!err.isEmpty()) {
-				api1->nvim_err_writeln(err.toLatin1());
-			}
-		});
 	}
 	emit neovimAttached(attached);
 	update();
@@ -230,6 +222,11 @@ void Shell::init()
 	if (m_options.enable_ext_popupmenu) {
 		options.insert("ext_popupmenu", true);
 	}
+	if (m_options.enable_ext_linegrid
+		&& m_nvim->hasUIOption("ext_linegrid")) {
+		// Modern Grid UI API is optionally enabled via cmdline
+		options.insert("ext_linegrid", true);
+	}
 	options.insert("rgb", true);
 
 	MsgpackRequest *req;
@@ -323,6 +320,13 @@ void Shell::handlePut(const QVariantList& args)
 	}
 
 	QString text = m_nvim->decode(args.at(0).toByteArray());
+	if (text.isEmpty() && m_cursor_pos.x() > 0 &&
+	    contents().constValue(m_cursor_pos.y(), m_cursor_pos.x() - 1).doubleWidth) {
+		// nvim will seek to the second cell of a wide char and put "",
+		// expecting the cursor position and cell style to be updated properly.
+		// Handle this case.
+		text = ' ';
+	}
 
 	if (!text.isEmpty()) {
 		int cols = put(text, m_cursor_pos.y(), m_cursor_pos.x(),
@@ -330,11 +334,8 @@ void Shell::handlePut(const QVariantList& args)
 				m_font_bold, m_font_italic,
 				m_font_underline, m_font_undercurl);
 		// Move cursor ahead
-		update(neovimCursorRect());
 		setNeovimCursor(m_cursor_pos.y(), m_cursor_pos.x()+cols);
-		update(neovimCursorRect());
 	}
-
 }
 
 /**
@@ -354,13 +355,9 @@ void Shell::handleScroll(const QVariantList& args)
 	}
 	qint64 count = args.at(0).toULongLong();
 
-	// Keep track of the cursor position, repaint
-	// over its old position after the scroll
 	if (m_scroll_region.contains(m_cursor_pos)) {
-		QPoint old_cursor_pos = m_cursor_pos;
-		old_cursor_pos.setY(old_cursor_pos.y()-count);
-		QRect cr = neovimCursorRect(old_cursor_pos);
-		update(cr);
+		// Schedule cursor region to be repainted
+		update(neovimCursorRect());
 	}
 
 	scrollShellRegion(m_scroll_region.top(), m_scroll_region.bottom(),
@@ -439,6 +436,9 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 			return;
 		}
 		setNeovimCursor(opargs.at(0).toULongLong(), opargs.at(1).toULongLong());
+		// @zhmars: On my system, call update(Qt::ImCursorRectangle) in function
+		// setNeovimCursor will cause typing lags
+		qApp->inputMethod()->update(Qt::ImCursorRectangle);
 	} else if (name == "highlight_set") {
 		if (opargs.size() < 1 && (QMetaType::Type)opargs.at(0).type() != QMetaType::QVariantMap) {
 			qWarning() << "Unexpected argument for redraw:" << name << opargs;
@@ -502,111 +502,101 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 			emit neovimSuspend();
 		}
 	} else if (name == "popupmenu_show") {
-		if (opargs.size() != 4
-				|| !opargs.at(1).canConvert<qint64>()
-				|| !opargs.at(2).canConvert<qint64>()
-				|| !opargs.at(3).canConvert<qint64>()) {
-			qWarning() << "Unexpected arguments for redraw:" << name << opargs;
-			return;
-		}
-
-		handlePopupMenuShow(opargs.at(0).toList(),
-				opargs.at(1).toLongLong(),
-				opargs.at(2).toLongLong(),
-				opargs.at(3).toLongLong());
+		handlePopupMenuShow(opargs);
 	} else if (name == "popupmenu_select") {
-		if (opargs.size() != 1 || !opargs.at(0).canConvert<qint64>()) {
-			qWarning() << "Unexpected arguments for redraw:" << name << opargs;
-			return;
-		}
-		// Neovim uses -1 for 'no selection' and so does Qt
-		handlePopupMenuSelect(opargs.at(0).toLongLong());
+		handlePopupMenuSelect(opargs);
 	} else if (name == "popupmenu_hide") {
 		m_pum.hide();
 	} else if (name == "mode_info_set") {
 		// TODO
+	} else if (name == "flush") {
+		// Do Nothing, a notification that nvim is done redrawing.
+	} else if (name == "grid_resize") {
+		handleGridResize(opargs);
 	} else if (name == "default_colors_set") {
-		// TODO
-	} else {
+		handleDefaultColorsSet(opargs);
+	} else if (name == "hl_attr_define") {
+		handleHighlightAttributeDefine(opargs);
+	} else if (name == "grid_line") {
+		handleGridLine(opargs);
+	} else if (name == "grid_clear") {
+		clearShell();
+	} else if (name == "grid_destroy") {
+		qDebug() << "Not implemented grid_destroy:" << opargs;
+	} else if (name == "grid_cursor_goto") {
+		handleGridCursorGoto(opargs);
+	} else if (name == "grid_scroll") {
+		handleGridScroll(opargs);
+	}
+	else {
 		qDebug() << "Received unknown redraw notification" << name << opargs;
 	}
 
 }
 
-/// Show the pum, with the given items, selecting the given item (idx)
-/// and place the pum at the given row/col
-void Shell::handlePopupMenuShow(const QVariantList& items,
-			int64_t selected, int64_t row, int64_t col)
+void Shell::handlePopupMenuShow(const QVariantList& opargs)
 {
+	// The 'popupmenu_show' API is not consistent across NeoVim versions!
+	// A 5th argument was introduced in neovim/neovim@16c3337
+	if (opargs.size() < 4
+		|| static_cast<QMetaType::Type>(opargs.at(0).type()) != QMetaType::QVariantList
+		|| !opargs.at(1).canConvert<int64_t>()
+		|| !opargs.at(2).canConvert<int64_t>()
+		|| !opargs.at(3).canConvert<int64_t>()) {
+		qWarning() << "Unexpected arguments for popupmenu_show:" << opargs;
+		return;
+	}
+	else if (opargs.size() >= 5 && !opargs.at(4).canConvert<int64_t>()) {
+		qWarning() << "Unexpected 5th argument for popupmenu_show:" << opargs.at(4);
+		return;
+	}
+
+	const QVariantList items = opargs.at(0).toList();
+	const int64_t selected = opargs.at(1).toULongLong();
+	const int64_t row = opargs.at(2).toULongLong();
+	const int64_t col = opargs.at(3).toULongLong();
+	//const int64_t grid = (opargs.size() < 5) ? 0 : opargs.at(4).toULongLong();
+
 	QList<PopupMenuItem> model;
-	foreach(const QVariant& v, items) {
+	for (const auto& v : items) {
 		QVariantList item = v.toList();
-		/// Item is (text, kind, extra, info)
+		// Item is (text, kind, extra, info)
 		QString text = item.value(0).toString();
-		if (item.isEmpty() || item.value(0).toString().isEmpty()
-				|| item.size() != 4) {
-			model.append({"", "", "", ""
-					});
-		} else {
-			model.append({
-					item.value(0).toString(),
-					item.value(1).toString(),
-					item.value(2).toString(),
-					item.value(3).toString()
-					});
+		if (item.size() < 4
+			|| item.isEmpty()
+			|| item.value(0).toString().isEmpty()) {
+
+			// Usually faster/smaller to init strings with {} instead of ""
+			model.append({ QString{}, QString{}, QString{}, QString{} });
+			continue;
 		}
+
+		model.append({
+			item.value(0).toString(),
+			item.value(1).toString(),
+			item.value(2).toString(),
+			item.value(3).toString() });
 	}
 
 	m_pum.setModel(new PopupMenuModel(model));
 
-	handlePopupMenuSelect(selected);
+	m_pum.setSelectedIndex(selected);
 
-	// By default the menu is as large as needed to fit all entries
-	int pum_width = m_pum.sizeHint().width();
-	int anchor_x = col*cellSize().width();
-
-	// If the menu width is larger than half the shell, make it half width
-	if (pum_width >= width()/2) {
-		pum_width = columns()/2*cellSize().width();
-		anchor_x = (columns()-1)/2*cellSize().width();
-	}
-
-	// If the menu still does not fit move the anchor to the left
-	if (pum_width > (columns()*cellSize().width()-anchor_x)) {
-		anchor_x = columns()*cellSize().width() - pum_width;
-	}
-
-	int pum_height = m_pum.sizeHint().height();
-	// By default the menu goes bellow the row
-	int anchor_y = (row+1)*cellSize().height();
-	if (row > rows() / 2) {
-		// TODO: leave a couple lines above/below?
-		if (pum_height > row*cellSize().height()) {
-			// The pum height is too large, move it to the top
-			anchor_y = 0;
-			pum_height = row*cellSize().height()-1;
-		} else {
-			// Display menu above the anchor row
-			anchor_y = row*cellSize().height()-pum_height-1;
-		}
-	} else {
-		// Display menu below anchor
-		if (pum_height > (rows()-row-1)*cellSize().height()) {
-			// Resize popupmenu to fit
-			pum_height = height() - anchor_y;
-		}
-	}
-
-	m_pum.setGeometry(anchor_x, anchor_y, pum_width, pum_height);
+	m_pum.setAnchor(row, col);
 	m_pum.updateGeometry();
 	m_pum.show();
 }
 
-void Shell::handlePopupMenuSelect(int64_t selected)
+void Shell::handlePopupMenuSelect(const QVariantList& opargs)
 {
-	auto idx = m_pum.model()->index(selected, 0);
-	m_pum.setCurrentIndex(idx);
-	m_pum.scrollTo(idx);
+	if (opargs.size() < 1
+		|| !opargs.at(0).canConvert<int64_t>()) {
+		qWarning() << "Unexpected arguments for popupmenu_select:" << opargs;
+		return;
+	}
+
+	// Neovim and Qt both use -1 for 'no selection'.
+	m_pum.setSelectedIndex(opargs.at(0).toLongLong());
 }
 
 void Shell::setNeovimCursor(quint64 row, quint64 col)
@@ -618,12 +608,19 @@ void Shell::setNeovimCursor(quint64 row, quint64 col)
 
 void Shell::handleModeChange(const QString& mode)
 {
+	auto old = m_insertMode;
+
 	// TODO: Implement visual aids for other modes
 	if (mode == "insert") {
 		m_insertMode = true;
 	} else {
 		m_insertMode = false;
 	}
+
+	// redraw the cursor
+	if (old != m_insertMode) {
+		update(neovimCursorRect());
+	}
 }
 
 void Shell::handleSetTitle(const QVariantList& opargs)
@@ -638,6 +635,9 @@ void Shell::handleSetTitle(const QVariantList& opargs)
 
 void Shell::handleBusy(bool busy)
 {
+	if (busy != m_neovimBusy) {
+		update(neovimCursorRect());
+	}
 	m_neovimBusy = busy;
 	if (busy) {
 		this->setCursor(Qt::WaitCursor);
@@ -729,6 +729,8 @@ void Shell::handleNeovimNotification(const QByteArray &name, const QVariantList&
 
 			qDebug() << "Neovim changed clipboard" << data << type << reg_name << clipboard;
 			QGuiApplication::clipboard()->setMimeData(clipData, clipboard);
+		} else if (guiEvName == "ShowContextMenu") {
+			emit neovimShowContextMenu();
 		}
 		return;
 	} else if (name != "redraw") {
@@ -799,6 +801,7 @@ void Shell::handleSetOption(const QString& name, const QVariant& value)
 	} else if (name == "termguicolors") {
 	} else if (name == "ext_cmdline") {
 	} else if (name == "ext_wildmenu") {
+	} else if (name == "ext_linegrid") {
 	} else {
 		qDebug() << "Received unknown option" << name << value;
 	}
@@ -810,6 +813,218 @@ void Shell::handleMouse(bool enabled)
 	m_mouseEnabled = enabled;
 }
 
+void Shell::handleGridResize(const QVariantList& opargs)
+{
+	if (opargs.size() < 3
+		|| !opargs.at(0).canConvert<qint64>()
+		|| !opargs.at(1).canConvert<qint64>()
+		|| !opargs.at(2).canConvert<qint64>()) {
+		qWarning() << "Unexpected arguments for grid_resize:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "grid" = opargs.at(0).toULongLong()
+
+	const uint64_t width = opargs.at(1).toULongLong();
+	const uint64_t height = opargs.at(2).toULongLong();
+
+	handleResize(width, height);
+}
+
+void Shell::handleDefaultColorsSet(const QVariantList& opargs)
+{
+	if (opargs.size() < 5
+		|| !opargs.at(0).canConvert<qint64>()
+		|| !opargs.at(1).canConvert<qint64>()
+		|| !opargs.at(2).canConvert<qint64>()
+		|| !opargs.at(3).canConvert<qint64>()
+		|| !opargs.at(4).canConvert<qint64>()) {
+		qWarning() << "Unexpected arguments for default_colors_set:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "cterm_fg" = opargs.at(3).toULongLong()
+	//     "cterm_bg" = opargs.at(4).toULongLong()
+
+	const uint64_t rgb_fg = opargs.at(0).toULongLong();
+	const uint64_t rgb_bg = opargs.at(1).toULongLong();
+	const uint64_t rgb_sp = opargs.at(2).toULongLong();
+
+	MsgpackRequest* getBackgroundMode{
+		m_nvim->api0()->vim_get_option(QString{ "background" }.toLatin1()) };
+
+	connect(getBackgroundMode, &MsgpackRequest::finished, this, &Shell::handleGetBackgroundOption);
+
+	const QColor foregroundColor = color(rgb_fg, QColor::Invalid);
+	const QColor backgroundColor = color(rgb_bg, QColor::Invalid);
+	const QColor specialColor = color(rgb_sp, QColor::Invalid);
+
+	// Update shellwidget default colors
+	setForeground(foregroundColor);
+	setBackground(backgroundColor);
+	setSpecial(specialColor);
+
+	// Cells drawn with the default colors require a re-paint
+	update();
+}
+
+void Shell::handleHighlightAttributeDefine(const QVariantList& opargs)
+{
+	if (opargs.size() < 4
+		|| !opargs.at(0).canConvert<qint64>()
+		|| static_cast<QMetaType::Type>(opargs.at(1).type()) != QMetaType::QVariantMap
+		|| static_cast<QMetaType::Type>(opargs.at(2).type()) != QMetaType::QVariantMap) {
+		qWarning() << "Unexpected arguments for hl_attr_define:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "cterm_attr" = opargs.at(2).toMap()
+
+	const uint64_t id = opargs.at(0).toULongLong();
+	const auto& rgb_attr = opargs.at(1).toMap();
+
+	const QColor foregroundColor = (rgb_attr.contains("foreground")) ?
+		color(rgb_attr.value("foreground").toLongLong(), QColor::Invalid) :
+		QColor::Invalid;
+
+	const QColor backgroundColor = (rgb_attr.contains("background")) ?
+		color(rgb_attr.value("background").toLongLong(), QColor::Invalid) :
+		QColor::Invalid;
+
+	const QColor specialColor = rgb_attr.contains(("special")) ?
+		color(rgb_attr.value("special").toLongLong(), QColor::Invalid) :
+		QColor::Invalid;
+
+	HighlightAttribute hl_attr {
+		foregroundColor,
+		backgroundColor,
+		specialColor,
+		rgb_attr.contains("reverse"),
+		rgb_attr.contains("italic"),
+		rgb_attr.contains("bold"),
+		rgb_attr.contains("underline"),
+		rgb_attr.contains("undercurl") };
+
+	m_highlightMap.insert(id, hl_attr);
+}
+
+
+void Shell::handleGridLine(const QVariantList& opargs)
+{
+	if (opargs.size() < 4
+		|| !opargs.at(0).canConvert<qint64>()
+		|| !opargs.at(1).canConvert<qint64>()
+		|| !opargs.at(2).canConvert<qint64>()
+		|| static_cast<QMetaType::Type>(opargs.at(3).type()) != QMetaType::QVariantList) {
+		qWarning() << "Unexpected arguments for grid_line:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "grid" = opargs.at(0).toULongLong()
+
+	const uint64_t row = opargs.at(1).toULongLong();
+	const uint64_t col_start = opargs.at(2).toULongLong();
+	const QVariantList& cells = opargs.at(3).toList();
+
+	// Last used hl_attr, default ctor triggers default highlight/style.
+	HighlightAttribute hl_attr;
+
+	uint64_t col_next = col_start;
+	for (const auto& cell : cells) {
+		const QVariantList& cellPropertyList = cell.toList();
+
+		QString text = m_nvim->decode(cellPropertyList[0].toByteArray());
+
+		// Optional highlight style, default is the last hl_attr.
+		if (cellPropertyList.size() > 1) {
+			const uint64_t hl_id = cellPropertyList[1].toULongLong();
+
+			// Entry for 'hl_id == 0' is intentionally absent.
+			// Any unknown key triggers the default highlight/style.
+			hl_attr = m_highlightMap.value(hl_id);
+		}
+
+		// Optional repeat count, default is 1.
+		uint64_t repeat = 1;
+		if (cellPropertyList.size() > 2) {
+			repeat = cellPropertyList[2].toULongLong();
+		}
+
+		// Send GUI updates to 'ShellWidget'.
+		for (uint64_t i=0;i<repeat;i++)
+		{
+			put(text, row, col_next,
+				hl_attr.GetForegroundColor(),
+				hl_attr.GetBackgroundColor(),
+				hl_attr.GetSpecialColor(),
+				hl_attr.IsBold(),
+				hl_attr.IsItalic(),
+				hl_attr.IsUnderline(),
+				hl_attr.IsUndercurl());
+
+			col_next++;
+		}
+	}
+}
+
+void Shell::handleGridCursorGoto(const QVariantList& opargs)
+{
+	if (opargs.size() < 3
+		|| !opargs.at(1).canConvert<quint64>()
+		|| !opargs.at(2).canConvert<quint64>()) {
+		qWarning() << "Unexpected arguments for grid_cursor_goto:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "grid" = opargs.at(0).toULongLong()
+
+	const uint64_t row = opargs.at(1).toULongLong();
+	const uint64_t column = opargs.at(2).toULongLong();
+
+	setNeovimCursor(row, column);
+	qApp->inputMethod()->update(Qt::ImCursorRectangle);
+}
+
+void Shell::handleGridScroll(const QVariantList& opargs)
+{
+	if (opargs.size() < 7
+		|| !opargs.at(0).canConvert<quint64>()
+		|| !opargs.at(1).canConvert<quint64>()
+		|| !opargs.at(2).canConvert<quint64>()
+		|| !opargs.at(3).canConvert<quint64>()
+		|| !opargs.at(4).canConvert<quint64>()
+		|| !opargs.at(5).canConvert<quint64>()
+		|| !opargs.at(6).canConvert<quint64>()) {
+		qWarning() << "Unexpected arguments for grid_scroll:" << opargs;
+		return;
+	}
+
+	// Unused parameters:
+	//     "grid" = opargs.at(0).toULongLong()
+	//     "cols" = opargs.at(6).toULongLong()
+
+	const uint64_t top = opargs.at(1).toULongLong();
+	const uint64_t bot = opargs.at(2).toULongLong();
+	const uint64_t left = opargs.at(3).toULongLong();
+	const uint64_t right = opargs.at(4).toULongLong();
+	const uint64_t rows = opargs.at(5).toULongLong();
+
+	m_scroll_region = QRect(QPoint(left, top), QPoint(right, bot));
+
+	if (m_scroll_region.contains(m_cursor_pos)) {
+		// Schedule cursor region to be repainted
+		update(neovimCursorRect());
+	}
+
+	scrollShellRegion(m_scroll_region.top(), m_scroll_region.bottom(),
+		m_scroll_region.left(), m_scroll_region.right(), rows);
+}
+
 void Shell::paintEvent(QPaintEvent *ev)
 {
 	if (!m_attached) {
@@ -822,7 +1037,7 @@ void Shell::paintEvent(QPaintEvent *ev)
 
 	// paint cursor - we are not actually using Neovim colors yet,
 	// just invert the shell colors by painting white with XoR
-	if (ev->region().contains(neovimCursorTopLeft())) {
+	if (!m_neovimBusy && ev->region().contains(neovimCursorTopLeft())) {
 		bool wide = contents().constValue(m_cursor_pos.y(),
 						m_cursor_pos.x()).doubleWidth;
 		QRect cursorRect(neovimCursorTopLeft(), cellSize());
@@ -833,8 +1048,13 @@ void Shell::paintEvent(QPaintEvent *ev)
 			cursorRect.setWidth(cursorRect.width()*2);
 		}
 		QPainter painter(this);
+		painter.setPen(m_cursor_color);
 		painter.setCompositionMode(QPainter::RasterOp_SourceXorDestination);
-		painter.fillRect(cursorRect, m_cursor_color);
+		if (hasFocus()) {
+			painter.fillRect(cursorRect, m_cursor_color);
+		} else {
+			painter.drawRect(cursorRect);
+		}
 	}
 }
 
@@ -940,7 +1160,7 @@ void Shell::mouseMoveEvent(QMouseEvent *ev)
 
 void Shell::wheelEvent(QWheelEvent *ev)
 {
-	if (!m_attached) {
+	if (!m_attached || !m_mouseEnabled) {
 		return;
 	}
 #ifdef Q_OS_MAC
@@ -1020,6 +1240,13 @@ void Shell::updateClientInfo()
 
 bool Shell::event(QEvent *event)
 {
+	if (event->type() == QEvent::KeyPress) {
+		QKeyEvent *ke = (QKeyEvent *)(event);
+		if (ke->key() == Qt::Key_Tab || ke->key() == Qt::Key_Backtab) {
+			keyPressEvent(ke);
+			return true;
+		}
+	}
 	if (event->type() == QEvent::WinIdChange){
 		updateWindowId();
 	}
@@ -1124,8 +1351,8 @@ void Shell::closeEvent(QCloseEvent *ev)
 void Shell::focusInEvent(QFocusEvent *ev)
 {
 	if (m_attached) {
-		// See neovim-qt/issues/329 the FocusGained key no longer exists, use autocmd instead
-		m_nvim->api0()->vim_command("if exists('#FocusGained') | doautocmd FocusGained | endif");
+		// Issue #329: The <FocusGained> key no longer exists, use autocmd instead.
+		m_nvim->api0()->vim_command("if exists('#FocusGained') | doautocmd <nomodeline> FocusGained | endif");
 	}
 	QWidget::focusInEvent(ev);
 }
@@ -1133,7 +1360,8 @@ void Shell::focusInEvent(QFocusEvent *ev)
 void Shell::focusOutEvent(QFocusEvent *ev)
 {
 	if (m_attached) {
-		m_nvim->api0()->vim_command("if exists('#FocusLost') | doautocmd FocusLost | endif");
+		// Issue #591: Option <nomodeline> prevents unwanted interaction, consistent with nvim.
+		m_nvim->api0()->vim_command("if exists('#FocusLost') | doautocmd <nomodeline> FocusLost | endif");
 	}
 	QWidget::focusOutEvent(ev);
 }
@@ -1191,8 +1419,8 @@ QVariant Shell::inputMethodQuery(Qt::InputMethodQuery query) const
 {
 	if ( query == Qt::ImFont) {
 		return font();
-	} else if ( query == Qt::ImMicroFocus ) {
-		return QRect(neovimCursorTopLeft(), QSize(0, cellSize().height()));
+	} else if ( query == Qt::ImMicroFocus || query == Qt::ImCursorRectangle ) {
+		return QRect(neovimCursorTopLeft(), cellSize());
 	}
 
 	return QVariant();
@@ -1431,4 +1659,18 @@ void Shell::handleShimError(quint32 msgid, quint64 fun, const QVariant& err)
 	qDebug() << "GUI shim error " << err;
 }
 
-} // Namespace
+void Shell::handleGetBackgroundOption(quint32 msgid, quint64 fun, const QVariant& val)
+{
+	const QString mode{ val.toString() };
+
+	if (mode == "dark" && getBackgroundType() != Background::Dark) {
+		setBackgroundType(Background::Dark);
+		update();
+	}
+	else if (mode == "light" && getBackgroundType() != Background::Light) {
+		setBackgroundType(Background::Light);
+		update();
+	}
+}
+
+} // namespace NeovimQt
