@@ -12,6 +12,7 @@
 #include <QMenu>
 #include "neovimconnector.h"
 #include "shellwidget/shellwidget.h"
+#include "highlight.h"
 #include "popupmenu.h"
 #include "popupmenumodel.h"
 
@@ -30,14 +31,10 @@ public:
 
 class ShellOptions {
 public:
-	ShellOptions() {
-		enable_ext_tabline = true;
-		enable_ext_popupmenu = true;
-		nvim_show_tabline = 1;
-	}
-	bool enable_ext_tabline;
-	int nvim_show_tabline;
-	bool enable_ext_popupmenu;
+	bool enable_ext_tabline{ true };
+	bool enable_ext_popupmenu{ true };
+	bool enable_ext_linegrid{ false };
+	int nvim_show_tabline{ 1 };
 };
 
 class Shell: public ShellWidget
@@ -73,6 +70,8 @@ signals:
 	/// as seen in Tab::tab.
 	void neovimTablineUpdate(int64_t curtab, QList<Tab> tabs);
 	void neovimShowtablineSet(int);
+	void neovimShowContextMenu();
+	void fontChanged();
 
 public slots:
 	void handleNeovimNotification(const QByteArray &name, const QVariantList& args);
@@ -91,6 +90,10 @@ protected slots:
 	void init();
 	void fontError(const QString& msg);
 	void updateWindowId();
+	void updateClientInfo();
+	void handleGinitError(quint32 msgid, quint64 fun, const QVariant& err);
+	void handleShimError(quint32 msgid, quint64 fun, const QVariant& err);
+	void handleGetBackgroundOption(quint32 msgid, quint64 fun, const QVariant& val);
 
 protected:
 	void tooltip(const QString& text);
@@ -125,14 +128,25 @@ protected:
 	virtual void handleBusy(bool);
 	virtual void handleSetOption(const QString& name, const QVariant& value);
 	void handleExtGuiOption(const QString& name, const QVariant& value);
-	virtual void handlePopupMenuShow(const QVariantList& items, int64_t selected,
-			int64_t row, int64_t col);
-	void handlePopupMenuSelect(int64_t selected);
+	virtual void handlePopupMenuShow(const QVariantList& opargs);
+	virtual void handlePopupMenuSelect(const QVariantList& opargs);
+	virtual void handleMouse(bool);
+
+	// Modern 'ext_linegrid' Grid UI Events
+	virtual void handleGridResize(const QVariantList& opargs);
+	virtual void handleDefaultColorsSet(const QVariantList& opargs);
+	virtual void handleHighlightAttributeDefine(const QVariantList& opargs);
+	virtual void handleGridLine(const QVariantList& opargs);
+	virtual void handleGridCursorGoto(const QVariantList& opargs);
+	virtual void handleGridScroll(const QVariantList& opargs);
 
 	void neovimMouseEvent(QMouseEvent *ev);
 	virtual void mousePressEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
 	virtual void mouseReleaseEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
 	virtual void mouseMoveEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
+	void bailoutIfinputBlocking();
+
+	QString neovimErrorToString(const QVariant& err);
 
 private slots:
         void setAttached(bool attached=true);
@@ -153,6 +167,9 @@ private:
 	QColor m_hg_foreground, m_hg_background, m_hg_special;
 	QColor m_cursor_color;
 
+	/// Modern 'ext_linegrid' highlight definition map
+	QMap<uint64_t, HighlightAttribute> m_highlightMap;
+
 	/// Cursor position in shell coordinates
 	QPoint m_cursor_pos;
 	bool m_cursor;
@@ -171,8 +188,18 @@ private:
 	// Properties
 	bool m_neovimBusy;
 	ShellOptions m_options;
-	PopupMenu m_pum;
+	PopupMenu m_pum{ this };
+	bool m_mouseEnabled;
 };
 
+class ShellRequestHandler: public QObject, public MsgpackRequestHandler
+{
+	Q_OBJECT
+public:
+	ShellRequestHandler(Shell *parent);
+	virtual void handleRequest(MsgpackIODevice* dev, quint32 msgid, const QByteArray& method, const QVariantList& args);
+};
+
+
 } // Namespace
 #endif
