@@ -3,6 +3,11 @@
 #include "shellwidget.h"
 #include "helpers.h"
 
+#if defined(Q_OS_WIN) && defined(USE_STATIC_QT)
+#include <QtPlugin>
+Q_IMPORT_PLUGIN (QWindowsIntegrationPlugin);
+#endif
+
 int main(int argc, char **argv)
 {
 	QApplication app(argc, argv);
