@@ -37,21 +37,34 @@ void ShellWidget::setDefaultFont()
 	setShellFont(DEFAULT_FONT, 11, -1, false, true);
 }
 
-bool ShellWidget::setShellFont(const QString& family, int ptSize, int weight, bool italic, bool force)
+bool ShellWidget::setShellFont(const QString& family, qreal ptSize, int weight, bool italic, bool force)
 {
-	QFont f(family, ptSize, weight, italic);
+	QFont f(family, -1, weight, italic);
+	// Issue #575: Clear style name. The KDE/Plasma theme plugin may set this
+	// but we want to match the family name with the bold/italic attributes.
+	f.setStyleName(QStringLiteral(""));
+
+	f.setPointSizeF(ptSize);
 	f.setStyleHint(QFont::TypeWriter, QFont::StyleStrategy(QFont::PreferDefault | QFont::ForceIntegerMetrics));
 	f.setFixedPitch(true);
 	f.setKerning(false);
 
+	// Issue #585 Error message "Unknown font:" for Neovim 0.4.2+.
+	// This case has always been hit, but results in user visible error messages for recent
+	// releases. It is safe to ignore this case, which occurs at startup time.
+	if (f.family().isEmpty()) {
+		return false;
+	}
+
 	QFontInfo fi(f);
 	if (fi.family().compare(f.family(), Qt::CaseInsensitive) != 0 &&
 			f.family().compare("Monospace", Qt::CaseInsensitive) != 0) {
 		emit fontError(QString("Unknown font: %1").arg(f.family()));
 		return false;
 	}
-	if ( !force ) {
-		if ( !fi.fixedPitch() ) {
+
+	if (!force) {
+		if (!fi.fixedPitch()) {
 			emit fontError(QString("%1 is not a fixed pitch font").arg(f.family()));
 			return false;
 		}
@@ -103,6 +116,9 @@ QSize ShellWidget::cellSize() const
 void ShellWidget::paintEvent(QPaintEvent *ev)
 {
 	QPainter p(this);
+
+	p.setClipping(true);
+
 	foreach(QRect rect, ev->region().rects()) {
 		int start_row = rect.top() / m_cellSize.height();
 		int end_row = rect.bottom() / m_cellSize.height();
@@ -110,21 +126,23 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 		int end_col = rect.right() / m_cellSize.width();
 
 		// Paint margins
-		if (end_col > m_contents.columns()) {
-			end_col = m_contents.columns();
+		if (end_col >= m_contents.columns()) {
+			end_col = m_contents.columns() - 1;
 		}
-		if (end_row > m_contents.rows()) {
-			end_col = m_contents.columns();
+		if (end_row >= m_contents.rows()) {
+			end_row = m_contents.rows() - 1;
 		}
 
 		// end_col/row is inclusive
-		for (int i=start_row; i<=end_row && i < m_contents.rows(); i++) {
-			for (int j=start_col; j<=end_col && j < m_contents.columns();
-					j++) {
+		for (int i=start_row; i<=end_row; i++) {
+			for (int j=end_col; j>=start_col; j--) {
 
 				const Cell& cell = m_contents.constValue(i,j);
 				int chars = cell.doubleWidth ? 2 : 1;
 				QRect r = absoluteShellRect(i, j, 1, chars);
+				QRect ovflw = absoluteShellRect(i, j, 1, chars + 1);
+
+				p.setClipRegion(ovflw);
 
 				if (j <= 0 || !contents().constValue(i, j-1).doubleWidth) {
 					// Only paint bg/fg if this is not the second cell
@@ -132,7 +150,7 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 					if (cell.backgroundColor.isValid()) {
 						p.fillRect(r, cell.backgroundColor);
 					} else {
-						p.fillRect(r, m_bgColor);
+						p.fillRect(r, background());
 					}
 
 					if (cell.c == ' ') {
@@ -142,7 +160,7 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 					if (cell.foregroundColor.isValid()) {
 						p.setPen(cell.foregroundColor);
 					} else {
-						p.setPen(m_fgColor);
+						p.setPen(foreground());
 					}
 
 					if (cell.bold || cell.italic) {
@@ -156,7 +174,7 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 
 					// Draw chars at the baseline
 					QPoint pos(r.left(), r.top()+m_ascent+(m_lineSpace / 2));
-					p.drawText(pos, QString(cell.c));
+					p.drawText(pos, QString::fromUcs4(&cell.c, 1));
 				}
 
 				// Draw "undercurl" at the bottom of the cell
@@ -165,18 +183,18 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 					if (cell.undercurl) {
 						if (cell.specialColor.isValid()) {
 							pen.setColor(cell.specialColor);
-						} else if (m_spColor.isValid()) {
-							pen.setColor(m_spColor);
+						} else if (special().isValid()) {
+							pen.setColor(special());
 						} else if (cell.foregroundColor.isValid()) {
 							pen.setColor(cell.foregroundColor);
 						} else {
-							pen.setColor(m_fgColor);
+							pen.setColor(foreground());
 						}
 					} else if (cell.underline) {
 						if (cell.foregroundColor.isValid()) {
 							pen.setColor(cell.foregroundColor);
 						} else {
-							pen.setColor(m_fgColor);
+							pen.setColor(foreground());
 						}
 					}
 
@@ -200,11 +218,13 @@ void ShellWidget::paintEvent(QPaintEvent *ev)
 		}
 	}
 
+	p.setClipping(false);
+
 	QRect shellArea = absoluteShellRect(0, 0,
 				m_contents.rows(), m_contents.columns());
 	QRegion margins = QRegion(rect()).subtracted(shellArea);
 	foreach(QRect margin, margins.intersected(ev->region()).rects()) {
-		p.fillRect(margin, m_bgColor);
+		p.fillRect(margin, background());
 	}
 
 #if 0
@@ -259,6 +279,24 @@ void ShellWidget::setBackground(const QColor& color)
 
 QColor ShellWidget::background() const
 {
+	// See 'default_colors_set' in Neovim ':help ui-linegrid'.
+	// QColor::Invalid indicates the default color (-1), which should be
+	// rendered as white or black based on Neovim 'background'.
+	if (!m_bgColor.isValid())
+	{
+		switch (m_background)
+		{
+			case Background::Light:
+				return Qt::white;
+
+			case Background::Dark:
+				return Qt::black;
+
+			default:
+				return Qt::black;
+		}
+	}
+
 	return m_bgColor;
 }
 
@@ -269,6 +307,22 @@ void ShellWidget::setForeground(const QColor& color)
 
 QColor ShellWidget::foreground() const
 {
+	// See ShellWidget::background() for more details.
+	if (!m_bgColor.isValid())
+	{
+		switch (m_background)
+		{
+			case Background::Light:
+				return Qt::black;
+
+			case Background::Dark:
+				return Qt::white;
+
+			default:
+				return Qt::white;
+		}
+	}
+
 	return m_fgColor;
 }
 
@@ -282,15 +336,6 @@ int ShellWidget::put(const QString& text, int row, int column,
 		QColor fg, QColor bg, QColor sp, bool bold, bool italic,
 		bool underline, bool undercurl)
 {
-	if (!fg.isValid()) {
-		fg = m_fgColor;
-	}
-	if (!bg.isValid()) {
-		bg = m_bgColor;
-	}
-	if (!sp.isValid()) {
-		sp = m_spColor;
-	}
 	int cols_changed = m_contents.put(text, row, column, fg, bg, sp,
 				bold, italic, underline, undercurl);
 	if (cols_changed > 0) {
@@ -354,9 +399,9 @@ QString ShellWidget::fontFamily() const
 {
 	return QFontInfo(font()).family();
 }
-int ShellWidget::fontSize() const
+qreal ShellWidget::fontSize() const
 {
-	return font().pointSize();
+	return font().pointSizeF();
 }
 
 int ShellWidget::rows() const
