@@ -17,13 +17,22 @@ class ShellWidget: public QWidget
 	Q_PROPERTY(QSize cellSize READ cellSize)
 public:
 	ShellWidget(QWidget *parent=0);
-	bool setShellFont(const QString& family, int ptSize, int weight = -1, bool italic = false, bool force = false);
+
+	/// Represents Neovim 'background' option. Used for rendering default theme colors (-1).
+	/// See 'default_colors_set' in Neovim ':help ui-linegrid' for more details.
+	enum class Background : int
+	{
+		Dark,
+		Light
+	};
+
+	bool setShellFont(const QString& family, qreal ptSize, int weight = -1, bool italic = false, bool force = false);
 
 	QColor background() const;
 	QColor foreground() const;
 	QColor special() const;
 	QString fontFamily() const;
-	int fontSize() const;
+	qreal fontSize() const;
 	static ShellWidget* fromFile(const QString& path);
 
 	int rows() const;
@@ -31,6 +40,11 @@ public:
 	QSize cellSize() const;
 	const ShellContents& contents() const;
 	QSize sizeHint() const Q_DECL_OVERRIDE;
+
+	Background getBackgroundType() const { return m_background; }
+
+	void setBackgroundType(Background type) { m_background = type; }
+
 signals:
 	void shellFontChanged();
 	void fontError(const QString& msg);
@@ -45,7 +59,7 @@ public slots:
 			bool bold=false, bool italic=false,
 			bool underline=false, bool undercurl=false);
 	void clearRow(int row);
-	void clearShell(QColor bg);
+	void clearShell(QColor bg = QColor::Invalid);
 	void clearRegion(int row0, int col0, int row1, int col1);
 	void scrollShell(int rows);
 	void scrollShellRegion(int row0, int row1, int col0,
@@ -66,6 +80,8 @@ private:
 	int m_ascent;
 	QColor m_bgColor, m_fgColor, m_spColor;
 	int m_lineSpace;
+
+	Background m_background{ Background::Dark };
 };
 
 #endif
