@@ -1,13 +1,18 @@
 #include <QtTest/QtTest>
 #include "cell.h"
 
+#if defined(Q_OS_WIN) && defined(USE_STATIC_QT)
+#include <QtPlugin>
+Q_IMPORT_PLUGIN (QWindowsIntegrationPlugin);
+#endif
+
 class Test: public QObject
 {
 	Q_OBJECT
 private slots:
 	void cellDefault() {
 		Cell c;
-		QCOMPARE(c.c, QChar(' '));
+		QCOMPARE(c.c, uint(' '));
 		// Default colors are invalid
 		QCOMPARE(c.foregroundColor, QColor());
 		QCOMPARE(c.backgroundColor, QColor());
@@ -37,7 +42,7 @@ private slots:
 	void cellWidth() {
 		Cell c;
 		QCOMPARE(c.doubleWidth, false);
-		c.setChar(QChar(27721));
+		c.setChar(27721);
 		QCOMPARE(c.doubleWidth, true);
 	}
 
