@@ -1,6 +1,11 @@
 #include <QtTest/QtTest>
 #include "shellwidget.h"
 
+#if defined(Q_OS_WIN) && defined(USE_STATIC_QT)
+#include <QtPlugin>
+Q_IMPORT_PLUGIN (QWindowsIntegrationPlugin);
+#endif
+
 class Test: public QObject
 {
 	Q_OBJECT
