@@ -3,7 +3,7 @@
 
 #include <QObject>
 #include <QTimer>
-#include "function.h"
+
 namespace NeovimQt {
 
 class MsgpackIODevice;
@@ -11,21 +11,21 @@ class MsgpackRequest: public QObject {
 	Q_OBJECT
 public:
 	MsgpackRequest(quint32 id, MsgpackIODevice *dev, QObject *parent=0);
-	void setFunction(Function::FunctionId);
-	Function::FunctionId function();
+	void setFunction(quint64);
+	quint64 function();
 	void setTimeout(int msec);
 	/** The identifier for this Msgpack request */
 	const quint32 id;
 signals:
-	void finished(quint32 msgid, Function::FunctionId fun, const QVariant& resp);
-	void error(quint32 msgid, Function::FunctionId fun, const QVariant& err);
+	void finished(quint32 msgid, quint64 fun, const QVariant& resp);
+	void error(quint32 msgid, quint64 fun, const QVariant& err);
 	void timeout(quint32 id);
 protected slots:
 	void requestTimeout();
 
 private:
 	MsgpackIODevice *m_dev;
-	Function::FunctionId m_function;
+	quint64 m_function;
 	QTimer m_timer;
 };
 } // Namespace
