@@ -24,13 +24,15 @@ private slots:
 			<< "third-party/DejaVuSansMono-Bold.ttf"
 			<< "third-party/DejaVuSansMono-BoldOblique.ttf";
 		foreach(QString path, fonts) {
-			QFontDatabase::addApplicationFont(path);
+		    QString abs_path_to_font(CMAKE_SOURCE_DIR);
+		    abs_path_to_font.append("/").append(path);
+		    QFontDatabase::addApplicationFont(abs_path_to_font);
 		}
 	}
 
 	void benchStart() {
 		QBENCHMARK {
-			NeovimConnector *c = NeovimConnector::spawn({"-u", "NORC"});
+			NeovimConnector *c = NeovimConnector::spawn({"-u", "NONE"});
 			QSignalSpy onReady(c, SIGNAL(ready()));
 			QVERIFY(onReady.isValid());
 			QVERIFY(SPYWAIT(onReady));
@@ -44,7 +46,7 @@ private slots:
 
 	void uiStart() {
 		QStringList args;
-		args << "-u" << "NORC";
+		args << "-u" << "NONE";
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		Shell *s = new Shell(c, ShellOptions());
 		QSignalSpy onAttached(s, SIGNAL(neovimAttached(bool)));
@@ -60,7 +62,7 @@ private slots:
 	}
 
 	void startVarsShellWidget() {
-		QStringList args = {"-u", "NORC"};
+		QStringList args = {"-u", "NONE"};
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		Shell *s = new Shell(c, ShellOptions());
 		QSignalSpy onAttached(s, SIGNAL(neovimAttached(bool)));
@@ -71,7 +73,7 @@ private slots:
 	}
 
 	void startVarsMainWindow() {
-		QStringList args = {"-u", "NORC"};
+		QStringList args = {"-u", "NONE"};
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		MainWindow *s = new MainWindow(c, ShellOptions());
 		s->show();
@@ -84,7 +86,7 @@ private slots:
 
 	void guiExtTablineSet() {
 		QStringList args;
-		args << "-u" << "NORC";
+		args << "-u" << "NONE";
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		Shell *s = new Shell(c, ShellOptions());
 		QSignalSpy onOptionSet(s, &Shell::neovimExtTablineSet);
@@ -95,7 +97,7 @@ private slots:
 	void gviminit() {
 		qputenv("GVIMINIT", "let g:test_gviminit = 1");
 		QStringList args;
-		args << "-u" << "NORC";
+		args << "-u" << "NONE";
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		Shell *s = new Shell(c, ShellOptions());
 
@@ -116,9 +118,11 @@ private slots:
 	void guiShimCommands() {
 		// This function needs to be able to find the GUI runtime
 		// plugin or this test WILL FAIL
-		QFileInfo fi = QFileInfo("src/gui/runtime");
+		QString path_to_src_runtime(CMAKE_SOURCE_DIR);
+		path_to_src_runtime.append("/src/gui/runtime");
+		QFileInfo fi = QFileInfo(path_to_src_runtime);
 		QVERIFY2(fi.exists(), "Unable to find GUI runtime");
-		QStringList args = {"-u", "NORC",
+		QStringList args = {"-u", "NONE",
 			"--cmd", "set rtp+=" + fi.absoluteFilePath()};
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		MainWindow *s = new MainWindow(c, ShellOptions());
@@ -210,8 +214,10 @@ private slots:
 		QFETCH(char, reg);
 		QFETCH(QByteArray, register_data);
 
-		QFileInfo fi = QFileInfo("src/gui/runtime");
-		QStringList args = {"-u", "NORC",
+		QString path_to_src_runtime(CMAKE_SOURCE_DIR);
+		path_to_src_runtime.append("/src/gui/runtime");
+		QFileInfo fi = QFileInfo(path_to_src_runtime);
+		QStringList args = {"-u", "NONE",
 			"--cmd", "set rtp+=" + fi.absoluteFilePath()};
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		MainWindow *s = new MainWindow(c, ShellOptions());
@@ -269,8 +275,10 @@ private slots:
 		QFETCH(char, reg);
 		QFETCH(QByteArray, register_data);
 
-		QFileInfo fi = QFileInfo("src/gui/runtime");
-		QStringList args = {"-u", "NORC",
+		QString path_to_src_runtime(CMAKE_SOURCE_DIR);
+		path_to_src_runtime.append("/src/gui/runtime");
+		QFileInfo fi = QFileInfo(path_to_src_runtime);
+		QStringList args = {"-u", "NONE",
 			"--cmd", "set rtp+=" + fi.absoluteFilePath()};
 		NeovimConnector *c = NeovimConnector::spawn(args);
 		MainWindow *s = new MainWindow(c, ShellOptions());
